@@ -810,11 +810,7 @@ impl GeP3 {
         let y_squared = y.square();
         let u = y_squared - FE_ONE;
         let v = (y_squared * FE_D) + FE_ONE;
-        let v_raise_3 = v.square() * v;
-        let v_raise_7 = v_raise_3.square() * v;
-        let uv7 = v_raise_7 * u; // Is this commutative? u comes second in the code, but not in the notation...
-
-        let mut x = uv7.pow25523() * v_raise_3 * u;
+        let mut x = (u * v).pow25523() * u;
 
         let vxx = x.square() * v;
         let check = vxx - u;
@@ -1035,23 +1031,6 @@ pub fn sc_reduce(s: &mut [u8]) {
     let s21: i64 = 2097151 & (load_3i(&s[55..58]) >> 1);
     let s22: i64 = 2097151 & (load_4i(&s[57..61]) >> 6);
     let s23: i64 = load_4i(&s[60..64]) >> 3;
-    let mut carry0: i64;
-    let mut carry1: i64;
-    let mut carry2: i64;
-    let mut carry3: i64;
-    let mut carry4: i64;
-    let mut carry5: i64;
-    let mut carry6: i64;
-    let mut carry7: i64;
-    let mut carry8: i64;
-    let mut carry9: i64;
-    let mut carry10: i64;
-    let mut carry11: i64;
-    let carry12: i64;
-    let carry13: i64;
-    let carry14: i64;
-    let carry15: i64;
-    let carry16: i64;
 
     s11 += s23 * 666643;
     s12 += s23 * 470296;
@@ -1095,38 +1074,38 @@ pub fn sc_reduce(s: &mut [u8]) {
     s10 += s18 * 136657;
     s11 -= s18 * 683901;
 
-    carry6 = (s6 + (1 << 20)) >> 21;
+    let mut carry6: i64 = (s6 + (1 << 20)) >> 21;
     s7 += carry6;
     s6 -= carry6 << 21;
-    carry8 = (s8 + (1 << 20)) >> 21;
+    let mut carry8: i64 = (s8 + (1 << 20)) >> 21;
     s9 += carry8;
     s8 -= carry8 << 21;
-    carry10 = (s10 + (1 << 20)) >> 21;
+    let mut carry10: i64 = (s10 + (1 << 20)) >> 21;
     s11 += carry10;
     s10 -= carry10 << 21;
-    carry12 = (s12 + (1 << 20)) >> 21;
+    let carry12: i64 = (s12 + (1 << 20)) >> 21;
     s13 += carry12;
     s12 -= carry12 << 21;
-    carry14 = (s14 + (1 << 20)) >> 21;
+    let carry14: i64 = (s14 + (1 << 20)) >> 21;
     s15 += carry14;
     s14 -= carry14 << 21;
-    carry16 = (s16 + (1 << 20)) >> 21;
+    let carry16: i64 = (s16 + (1 << 20)) >> 21;
     s17 += carry16;
     s16 -= carry16 << 21;
 
-    carry7 = (s7 + (1 << 20)) >> 21;
+    let mut carry7: i64 = (s7 + (1 << 20)) >> 21;
     s8 += carry7;
     s7 -= carry7 << 21;
-    carry9 = (s9 + (1 << 20)) >> 21;
+    let mut carry9: i64 = (s9 + (1 << 20)) >> 21;
     s10 += carry9;
     s9 -= carry9 << 21;
-    carry11 = (s11 + (1 << 20)) >> 21;
+    let mut carry11: i64 = (s11 + (1 << 20)) >> 21;
     s12 += carry11;
     s11 -= carry11 << 21;
-    carry13 = (s13 + (1 << 20)) >> 21;
+    let carry13: i64 = (s13 + (1 << 20)) >> 21;
     s14 += carry13;
     s13 -= carry13 << 21;
-    carry15 = (s15 + (1 << 20)) >> 21;
+    let carry15: i64 = (s15 + (1 << 20)) >> 21;
     s16 += carry15;
     s15 -= carry15 << 21;
 
@@ -1173,13 +1152,13 @@ pub fn sc_reduce(s: &mut [u8]) {
     s5 -= s12 * 683901;
     s12 = 0;
 
-    carry0 = (s0 + (1 << 20)) >> 21;
+    let mut carry0: i64 = (s0 + (1 << 20)) >> 21;
     s1 += carry0;
     s0 -= carry0 << 21;
-    carry2 = (s2 + (1 << 20)) >> 21;
+    let mut carry2: i64 = (s2 + (1 << 20)) >> 21;
     s3 += carry2;
     s2 -= carry2 << 21;
-    carry4 = (s4 + (1 << 20)) >> 21;
+    let mut carry4: i64 = (s4 + (1 << 20)) >> 21;
     s5 += carry4;
     s4 -= carry4 << 21;
     carry6 = (s6 + (1 << 20)) >> 21;
@@ -1192,13 +1171,13 @@ pub fn sc_reduce(s: &mut [u8]) {
     s11 += carry10;
     s10 -= carry10 << 21;
 
-    carry1 = (s1 + (1 << 20)) >> 21;
+    let mut carry1: i64 = (s1 + (1 << 20)) >> 21;
     s2 += carry1;
     s1 -= carry1 << 21;
-    carry3 = (s3 + (1 << 20)) >> 21;
+    let mut carry3: i64 = (s3 + (1 << 20)) >> 21;
     s4 += carry3;
     s3 -= carry3 << 21;
-    carry5 = (s5 + (1 << 20)) >> 21;
+    let mut carry5: i64 = (s5 + (1 << 20)) >> 21;
     s6 += carry5;
     s5 -= carry5 << 21;
     carry7 = (s7 + (1 << 20)) >> 21;
@@ -1368,63 +1347,17 @@ pub fn sc_muladd(s: &mut [u8], a: &[u8], b: &[u8], c: &[u8]) {
     let c9 = 2097151 & (load_4i(&c[23..27]) >> 5);
     let c10 = 2097151 & (load_3i(&c[26..29]) >> 2);
     let c11 = load_4i(&c[28..32]) >> 7;
-    let mut s0: i64;
-    let mut s1: i64;
-    let mut s2: i64;
-    let mut s3: i64;
-    let mut s4: i64;
-    let mut s5: i64;
-    let mut s6: i64;
-    let mut s7: i64;
-    let mut s8: i64;
-    let mut s9: i64;
-    let mut s10: i64;
-    let mut s11: i64;
-    let mut s12: i64;
-    let mut s13: i64;
-    let mut s14: i64;
-    let mut s15: i64;
-    let mut s16: i64;
-    let mut s17: i64;
-    let mut s18: i64;
-    let mut s19: i64;
-    let mut s20: i64;
-    let mut s21: i64;
-    let mut s22: i64;
-    let mut s23: i64;
-    let mut carry0: i64;
-    let mut carry1: i64;
-    let mut carry2: i64;
-    let mut carry3: i64;
-    let mut carry4: i64;
-    let mut carry5: i64;
-    let mut carry6: i64;
-    let mut carry7: i64;
-    let mut carry8: i64;
-    let mut carry9: i64;
-    let mut carry10: i64;
-    let mut carry11: i64;
-    let mut carry12: i64;
-    let mut carry13: i64;
-    let mut carry14: i64;
-    let mut carry15: i64;
-    let mut carry16: i64;
-    let carry17: i64;
-    let carry18: i64;
-    let carry19: i64;
-    let carry20: i64;
-    let carry21: i64;
-    let carry22: i64;
 
-    s0 = c0 + a0 * b0;
-    s1 = c1 + a0 * b1 + a1 * b0;
-    s2 = c2 + a0 * b2 + a1 * b1 + a2 * b0;
-    s3 = c3 + a0 * b3 + a1 * b2 + a2 * b1 + a3 * b0;
-    s4 = c4 + a0 * b4 + a1 * b3 + a2 * b2 + a3 * b1 + a4 * b0;
-    s5 = c5 + a0 * b5 + a1 * b4 + a2 * b3 + a3 * b2 + a4 * b1 + a5 * b0;
-    s6 = c6 + a0 * b6 + a1 * b5 + a2 * b4 + a3 * b3 + a4 * b2 + a5 * b1 + a6 * b0;
-    s7 = c7 + a0 * b7 + a1 * b6 + a2 * b5 + a3 * b4 + a4 * b3 + a5 * b2 + a6 * b1 + a7 * b0;
-    s8 = c8
+    let mut s0: i64 = c0 + a0 * b0;
+    let mut s1: i64 = c1 + a0 * b1 + a1 * b0;
+    let mut s2: i64 = c2 + a0 * b2 + a1 * b1 + a2 * b0;
+    let mut s3: i64 = c3 + a0 * b3 + a1 * b2 + a2 * b1 + a3 * b0;
+    let mut s4: i64 = c4 + a0 * b4 + a1 * b3 + a2 * b2 + a3 * b1 + a4 * b0;
+    let mut s5: i64 = c5 + a0 * b5 + a1 * b4 + a2 * b3 + a3 * b2 + a4 * b1 + a5 * b0;
+    let mut s6: i64 = c6 + a0 * b6 + a1 * b5 + a2 * b4 + a3 * b3 + a4 * b2 + a5 * b1 + a6 * b0;
+    let mut s7: i64 =
+        c7 + a0 * b7 + a1 * b6 + a2 * b5 + a3 * b4 + a4 * b3 + a5 * b2 + a6 * b1 + a7 * b0;
+    let mut s8: i64 = c8
         + a0 * b8
         + a1 * b7
         + a2 * b6
@@ -1434,7 +1367,7 @@ pub fn sc_muladd(s: &mut [u8], a: &[u8], b: &[u8], c: &[u8]) {
         + a6 * b2
         + a7 * b1
         + a8 * b0;
-    s9 = c9
+    let mut s9: i64 = c9
         + a0 * b9
         + a1 * b8
         + a2 * b7
@@ -1445,7 +1378,7 @@ pub fn sc_muladd(s: &mut [u8], a: &[u8], b: &[u8], c: &[u8]) {
         + a7 * b2
         + a8 * b1
         + a9 * b0;
-    s10 = c10
+    let mut s10: i64 = c10
         + a0 * b10
         + a1 * b9
         + a2 * b8
@@ -1457,7 +1390,7 @@ pub fn sc_muladd(s: &mut [u8], a: &[u8], b: &[u8], c: &[u8]) {
         + a8 * b2
         + a9 * b1
         + a10 * b0;
-    s11 = c11
+    let mut s11: i64 = c11
         + a0 * b11
         + a1 * b10
         + a2 * b9
@@ -1470,7 +1403,7 @@ pub fn sc_muladd(s: &mut [u8], a: &[u8], b: &[u8], c: &[u8]) {
         + a9 * b2
         + a10 * b1
         + a11 * b0;
-    s12 = a1 * b11
+    let mut s12: i64 = a1 * b11
         + a2 * b10
         + a3 * b9
         + a4 * b8
@@ -1481,7 +1414,7 @@ pub fn sc_muladd(s: &mut [u8], a: &[u8], b: &[u8], c: &[u8]) {
         + a9 * b3
         + a10 * b2
         + a11 * b1;
-    s13 = a2 * b11
+    let mut s13: i64 = a2 * b11
         + a3 * b10
         + a4 * b9
         + a5 * b8
@@ -1491,86 +1424,87 @@ pub fn sc_muladd(s: &mut [u8], a: &[u8], b: &[u8], c: &[u8]) {
         + a9 * b4
         + a10 * b3
         + a11 * b2;
-    s14 =
+    let mut s14: i64 =
         a3 * b11 + a4 * b10 + a5 * b9 + a6 * b8 + a7 * b7 + a8 * b6 + a9 * b5 + a10 * b4 + a11 * b3;
-    s15 = a4 * b11 + a5 * b10 + a6 * b9 + a7 * b8 + a8 * b7 + a9 * b6 + a10 * b5 + a11 * b4;
-    s16 = a5 * b11 + a6 * b10 + a7 * b9 + a8 * b8 + a9 * b7 + a10 * b6 + a11 * b5;
-    s17 = a6 * b11 + a7 * b10 + a8 * b9 + a9 * b8 + a10 * b7 + a11 * b6;
-    s18 = a7 * b11 + a8 * b10 + a9 * b9 + a10 * b8 + a11 * b7;
-    s19 = a8 * b11 + a9 * b10 + a10 * b9 + a11 * b8;
-    s20 = a9 * b11 + a10 * b10 + a11 * b9;
-    s21 = a10 * b11 + a11 * b10;
-    s22 = a11 * b11;
-    s23 = 0;
+    let mut s15: i64 =
+        a4 * b11 + a5 * b10 + a6 * b9 + a7 * b8 + a8 * b7 + a9 * b6 + a10 * b5 + a11 * b4;
+    let mut s16: i64 = a5 * b11 + a6 * b10 + a7 * b9 + a8 * b8 + a9 * b7 + a10 * b6 + a11 * b5;
+    let mut s17: i64 = a6 * b11 + a7 * b10 + a8 * b9 + a9 * b8 + a10 * b7 + a11 * b6;
+    let mut s18: i64 = a7 * b11 + a8 * b10 + a9 * b9 + a10 * b8 + a11 * b7;
+    let mut s19: i64 = a8 * b11 + a9 * b10 + a10 * b9 + a11 * b8;
+    let mut s20: i64 = a9 * b11 + a10 * b10 + a11 * b9;
+    let mut s21: i64 = a10 * b11 + a11 * b10;
+    let mut s22: i64 = a11 * b11;
+    let mut s23: i64 = 0;
 
-    carry0 = (s0 + (1 << 20)) >> 21;
+    let mut carry0: i64 = (s0 + (1 << 20)) >> 21;
     s1 += carry0;
     s0 -= carry0 << 21;
-    carry2 = (s2 + (1 << 20)) >> 21;
+    let mut carry2: i64 = (s2 + (1 << 20)) >> 21;
     s3 += carry2;
     s2 -= carry2 << 21;
-    carry4 = (s4 + (1 << 20)) >> 21;
+    let mut carry4: i64 = (s4 + (1 << 20)) >> 21;
     s5 += carry4;
     s4 -= carry4 << 21;
-    carry6 = (s6 + (1 << 20)) >> 21;
+    let mut carry6: i64 = (s6 + (1 << 20)) >> 21;
     s7 += carry6;
     s6 -= carry6 << 21;
-    carry8 = (s8 + (1 << 20)) >> 21;
+    let mut carry8: i64 = (s8 + (1 << 20)) >> 21;
     s9 += carry8;
     s8 -= carry8 << 21;
-    carry10 = (s10 + (1 << 20)) >> 21;
+    let mut carry10: i64 = (s10 + (1 << 20)) >> 21;
     s11 += carry10;
     s10 -= carry10 << 21;
-    carry12 = (s12 + (1 << 20)) >> 21;
+    let mut carry12: i64 = (s12 + (1 << 20)) >> 21;
     s13 += carry12;
     s12 -= carry12 << 21;
-    carry14 = (s14 + (1 << 20)) >> 21;
+    let mut carry14: i64 = (s14 + (1 << 20)) >> 21;
     s15 += carry14;
     s14 -= carry14 << 21;
-    carry16 = (s16 + (1 << 20)) >> 21;
+    let mut carry16: i64 = (s16 + (1 << 20)) >> 21;
     s17 += carry16;
     s16 -= carry16 << 21;
-    carry18 = (s18 + (1 << 20)) >> 21;
+    let carry18: i64 = (s18 + (1 << 20)) >> 21;
     s19 += carry18;
     s18 -= carry18 << 21;
-    carry20 = (s20 + (1 << 20)) >> 21;
+    let carry20: i64 = (s20 + (1 << 20)) >> 21;
     s21 += carry20;
     s20 -= carry20 << 21;
-    carry22 = (s22 + (1 << 20)) >> 21;
+    let carry22: i64 = (s22 + (1 << 20)) >> 21;
     s23 += carry22;
     s22 -= carry22 << 21;
 
-    carry1 = (s1 + (1 << 20)) >> 21;
+    let mut carry1: i64 = (s1 + (1 << 20)) >> 21;
     s2 += carry1;
     s1 -= carry1 << 21;
-    carry3 = (s3 + (1 << 20)) >> 21;
+    let mut carry3: i64 = (s3 + (1 << 20)) >> 21;
     s4 += carry3;
     s3 -= carry3 << 21;
-    carry5 = (s5 + (1 << 20)) >> 21;
+    let mut carry5: i64 = (s5 + (1 << 20)) >> 21;
     s6 += carry5;
     s5 -= carry5 << 21;
-    carry7 = (s7 + (1 << 20)) >> 21;
+    let mut carry7: i64 = (s7 + (1 << 20)) >> 21;
     s8 += carry7;
     s7 -= carry7 << 21;
-    carry9 = (s9 + (1 << 20)) >> 21;
+    let mut carry9: i64 = (s9 + (1 << 20)) >> 21;
     s10 += carry9;
     s9 -= carry9 << 21;
-    carry11 = (s11 + (1 << 20)) >> 21;
+    let mut carry11: i64 = (s11 + (1 << 20)) >> 21;
     s12 += carry11;
     s11 -= carry11 << 21;
-    carry13 = (s13 + (1 << 20)) >> 21;
+    let mut carry13: i64 = (s13 + (1 << 20)) >> 21;
     s14 += carry13;
     s13 -= carry13 << 21;
-    carry15 = (s15 + (1 << 20)) >> 21;
+    let mut carry15: i64 = (s15 + (1 << 20)) >> 21;
     s16 += carry15;
     s15 -= carry15 << 21;
-    carry17 = (s17 + (1 << 20)) >> 21;
+    let carry17: i64 = (s17 + (1 << 20)) >> 21;
     s18 += carry17;
     s17 -= carry17 << 21;
-    carry19 = (s19 + (1 << 20)) >> 21;
+    let carry19: i64 = (s19 + (1 << 20)) >> 21;
     s20 += carry19;
     s19 -= carry19 << 21;
-    carry21 = (s21 + (1 << 20)) >> 21;
+    let carry21: i64 = (s21 + (1 << 20)) >> 21;
     s22 += carry21;
     s21 -= carry21 << 21;
 
